import pyhop

###### CLASSES #####

class Location:
	def __init__(self,name):
		self.name = name
		self.pathto = []
		self.roadto = []
		self.metroto = []

	def __str__(self):
        	return str(self.name)

	def connect_by_path(self,to):
		self.pathto = self.pathto + [to.name]
		to.pathto = to.pathto + [self.name]

	def connect_by_road(self,to):
		self.roadto = self.roadto + [to.name]
		to.roadto = to.roadto + [self.name]

	def connect_by_metro(self,to):
		self.metroto = self.metroto + [to.name]
		to.metroto = to.metroto + [self.name]

class Person:
	def __init__(self,name,startloc,goalloc):
		self.name = name
		self.location = startloc
		self.goal = goalloc

	def __str__(self):
        	return str(self.name) + '(' + str(self.location) + ')'

class Car:
	def __init__(self,name,startloc,owner):
		self.name = name
		self.location = startloc
		self.owner = owner

	def __str__(self):
        	return str(self.name) + '(' + str(self.location) + ')'

class Metro:
	def __init__(self,line,startloc):
		self.line = line
		self.location = startloc
		self.aboard = []

	def __str__(self):
        	return str(self.line) + '(' + str(self.location) + ')'

	def board(self,who):
		self.aboard = self.aboard + [who]


######### OPERATORS ########

def walk(state,name,frm,to):
	person = state.people[name]
	print 'walk ' + str(person) + ':' + frm + '->'+to
	
	if person.location == frm and to in locations[frm].pathto:
		person.location = to
		print 'walk reached ' + to
		return state
	else:
		return False

pyhop.declare_operators(walk)

def enter_metro(state,line,at,name):
	if state.people[name].location == at and state.vehicles[line].location == at:
		state.vehicles[line].board(name)
		return state
	else:
		return False

pyhop.declare_operators(enter_metro)

def move_metro(state,line,frm,to):
	metro = state.vehicles[line]
	print 'move_metro ' + str(metro) + ':' + frm + '->'+to
	if metro.location == to:
		return state
	if metro.location == frm and to in locations[frm].metroto:
		metro.location = to
		print 'metro reached ' + to
		return state
	else:
		return False

pyhop.declare_operators(move_metro)

def leave_metro(state,line,at,name):
	if name in state.vehicles[line].aboard and state.vehicles[line].location == at:
		state.people[name].location = at
		return state
	else:
		return False

pyhop.declare_operators(leave_metro)
	
	
		

######### METHODS ########

def problem(state,dummy):
	sequence = []
	for person in state.people:
		sequence = sequence + [('travel',person,state.people[person].location,state.people[person].goal,[])]
	return [sequence]

pyhop.declare_methods('problem',problem)

def travel(state,name,frm,to,visited):
	person = state.people[name]
	print 'travel ' + str(person) + ':' + frm + '->'+to 
	if frm in visited:
		print frm + ' already visited!'
		return False
	visited = visited + [frm]
	if to in locations[frm].pathto:
		return [[('walk',name,frm,to)]]
	elif to in locations[frm].metroto:
		return [[('ride_metro','C',name,frm,to)]]
	else:
		branch = []
		for intermediate in locations[frm].pathto:
			branch = branch + [[('walk',name,frm,intermediate),('travel',name,intermediate,to,visited)]]
		return branch	
	return False

pyhop.declare_methods('travel',travel)

def ride_metro(state,line,name,frm,to):
	person = state.people[name]
	print 'ride_metro ' + str(person) + ':' + frm + '->'+to
	return [[('wait_for_metro',line,frm),('enter_metro',line,frm,name),('travel_metro',line,frm,to,[]),('leave_metro',line,to,name)]]

pyhop.declare_methods('ride_metro',ride_metro)

def wait_for_metro(state,line,frm):
	print 'wait_for_metro ' + line + ':' + frm
	if state.vehicles[line].location == frm:
		return [[]]
	return [[('travel_metro',line,state.vehicles[line].location,frm,[])]]

pyhop.declare_methods('wait_for_metro',wait_for_metro)

def travel_metro(state,line,frm,to,visited):
	metro = state.vehicles[line]
	print 'travel_metro ' + str(metro) + ':' + frm + '->'+to 
	if frm in visited:
		print frm + ' already visited!'
		return False
	visited = visited + [frm]
	if to in locations[frm].metroto:
		return [[('move_metro',line,frm,to)]]
	else:
		branch = []
		for intermediate in locations[frm].metroto:
			branch = branch + [[('move_metro',line,frm,intermediate),('move_metro',line,intermediate,to,visited)]]
		return branch	
	return False

pyhop.declare_methods('travel_metro',travel_metro)

######### CITY GRAPH ########

locations = {
	'Karlak':Location('Karlak'),
	'IPak':Location('IPak'),
	'Staromak':Location('Staromak'),
	'Narodni':Location('Narodni'),
	'Budejovicka':Location('Budejovicka')
	}

locations['Karlak'].connect_by_path(locations['IPak'])
locations['Karlak'].connect_by_path(locations['Narodni'])
locations['Narodni'].connect_by_path(locations['Staromak'])

locations['IPak'].connect_by_metro(locations['Budejovicka'])

######### INIT ########

state1 = pyhop.State('init')
state1.people = {
	'ja':Person('ja','Karlak','Staromak'),
	'Adam':Person('Adam','Karlak','Budejovicka')
	}
state1.vehicles = {'C':Metro('C','Budejovicka')}

pyhop.print_operators()
pyhop.print_methods()

print 'run! ' + str(state1.people)

pyhop.pyhop(state1,[[('problem','dummy')]],verbose=3)


	
